﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Security.Cryptography.X509Certificates;

namespace WebFormApp.HttpModules
{
    public class AuthenticationModule : IHttpModule
    {
        private static readonly log4net.ILog _logger = log4net.LogManager.GetLogger(typeof(AuthenticationModule));

        public void Dispose()
        {
        }

        public void Init(HttpApplication context)
        {
            if (_logger.IsInfoEnabled) _logger.Info("Init");

            // Add handler for the Authenticate event
            context.AuthenticateRequest += AuthenticateRequestHandler;
        }

        void AuthenticateRequestHandler(object sender, EventArgs e)
        {
            if (_logger.IsInfoEnabled) _logger.Info("AuthenticateRequestHandler");

            HttpApplication app = (HttpApplication)sender;
            HttpContext context = app.Context;

            X509Certificate2 userCert = null;

            // If client certificate is presented...
            if (context.Request.ClientCertificate.IsPresent)
            {
                userCert = new X509Certificate2(context.Request.ClientCertificate.Certificate);
                X500DistinguishedName dn = userCert.SubjectName;
                string userDn = dn.Name.Trim().ToUpper();

                if (_logger.IsInfoEnabled) _logger.Info(string.Format("OnAuthorization : UserDn = {0}", userDn));

                if (userDn != "CN=JOHN DOE")
                {
                    throw new HttpException(401, "Invalid Client Certificate");
                }
            }
            else
            {
                throw new HttpException(401, "Client Certificate Missing");
            }
        }
    }
}